;---------------------------------------------------------------------------------------------------------
;ATTENTION: This code has been developed to be used with the CBM Program Studio compiler
;
;Tip: BUILD (the entire project) using: ctrl+F5
;---------------------------------------------------------------------------------------------------------


;===============================================================================                
;                   z e r o - p a g e   v a r i a b l e s 
;-------------------------------------------------------------------------------
;the zeropage has plenty of room (even with kernal enabled) for fast variables
;-------------------------------------------------------------------------------
;undefined       = $02           ;

;undefined       = $10           ;
;undefined       = $11           ;
;undefined       = $12
;undefined       = $13

;undefined       = $14           ;
;undefined       = $15           ;
;undefined       = $16           ;
;undefined       = $17           ;
;undefined       = $18          ;
;undefined       = $19           ; 

POINTER_BUF     = $20           ;$20 & $21
;POINTER_BUF_H  = $21           ;

CHAR_ADDR       = $22           ;$22 & $23
;CHAR_ADDR_H    = $23           ;
CURSOR_ADDR     = $24           ;$24 & $25
;CURSOR_ADDR_H  = $25           ;
STR_ADDR        = $26           ;$26 & $27
;STR_ADDR_H     = $27           ;
;undefined      = $2B           ;
;undefined      = $2C           ;
MEM_ADDR        = $30           ;$30 & $31
;MEM_ADDR_H     = $31           ;

TMR_A_LO = $FB           ;
TMR_A_HI = $FC           ;
TMR_B_LO = $FD           ;
TMR_B_HI = $FE           ;

;===============================================================================                
;                c a s s e t t e b u f f e r   v a r i a b l e s 
;-------------------------------------------------------------------------------
;the casette buffer ranges from: 033C-03FB (plenty of room for slower variables)
;-------------------------------------------------------------------------------



;===============================================================================                
;                             C O N S T A N T S
;===============================================================================                


DATA_DIR_6510 = $00     ;the MOS6510 data direction register of the peripheral IO pins (P7-0)
DATA_BIT_6510 = $01     ;the MOS6510 value f the bits of the peripheral IO pins (P7-0)


;COLOR          = $0268         ;current characxter color
;GDCOL          = $0287         ;backgorund color under cursor
BORDER          = $D020         ;bordercolour
BACKGROUND      = $D021         ;background-0
BACKGROUND1     = $D022         ;background-1
BACKGROUND2     = $D023         ;background-2
BACKGROUND3     = $D024         ;background-3
DATAPORTB       = $DC01         ;Data Port B (keyboard, joystick, paddles),:gameport 1        

;-------------------------------------------------------------------------------


VIDEO_BASE      = $4000 ;the start of the video mem we want to write too
COLORSCREEN     = $D800                 ;location of the color screen memory (this value is fixed)
CHARSCREEN      = VIDEO_BASE + $3C00    ;location of the character screen memory

;-------------------------------------------------------------------------------

; C64 Joystick related constants
; ------------------------------
JOYPORT2        = $DC00   ;Joystick in PORT-2 @ $DC00 (if = 1 then button not pressed)
JOYPORT1        = $DC01   ;Joystick in PORT-1 @ $DC01 (if = 1 then button not pressed)
JOY_UP          = $01     ;bit 0 - $01 - 0000 0001 = joystick up/forward
JOY_DOWN        = $02     ;bit 1 - $02 - 0000 0010 = down/backward
JOY_LEFT        = $04     ;bit 2 - $04 - 0000 0100 = left
JOY_RIGHT       = $08     ;bit 3 - $08 - 0000 1000 = right
JOY_FIRE        = $10     ;bit 4 - $10 - 0001 0000 = fire

;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                                           BASIC STARTING ROUTINE 
;      now the program will start after typing the command "RUN" within basic (just like a real program)
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
; 10 SYS (2080)

*=$0801

        BYTE    $0E, $08, $0A, $00, $9E, $20, $28,  $32, $30, $38, $30, $29, $00, $00, $00

*=$0820

        JMP INITIALIZE




;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                                            INTERRUPTS
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

; interrupts are not allowed !!!

;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                                                  INITIALIZE
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

INITIALIZE      SEI                     ;prevent all interrupts that may have been enabled


                ;for a correct measurement we must know the system frequency. Fortunately, there are
                ;only two flavours, which we can detect by the generated video format, PAL or NTSC.
                ;So by detecting if we have a PAL or an NTSC system we know which timers values we
                ;should use in order to create the perfect 1 second timer.

PALNTSC_DETECTION                       ;PAL/NTSC detect - 0 = PAL, non-zero = NTSC
PN_WAIT         LDA $D012               ;Source: https://csdb.dk/forums/?roomid=10&topicid=3352
                BNE PN_WAIT             ;wait for rasterline 0 or 256
PN_WAIT1        LDA $D011               ;is rasterbeam in the area
                BPL PN_WAIT1            ;0-255? if yes, wait
PN_WAIT2        LDY #$00                ;
PN_SYNCH1       LDA $D012               ;
                CMP #$37                ;top pal rasterline
                BNE PN_SYNCH1           ;
                LDA $D012               ;if next is 0, then pal
PN_SYNCH2       CMP $D012               ;
                BEQ PN_SYNCH2           ;
                LDA $D012               ;
                BEQ PAL_VALUES          ;check result, if 0 it means we have a PAL system

NTSC_VALUES     LDA #$00                ;1 second requires 1022727 cycles of the clock (on an NTSC system)
                STA TMR_A_HI            ;1022727 =   256 *  3995
                LDA #$FF                ;         $00FF * $0F9B
                STA TMR_A_LO            ;       timer-A * timer-B
                LDA #$0F                ;
                STA TMR_B_HI            ;
                LDA #$9B                ;
                STA TMR_B_LO            ;
                LDA #$5C                ;
                STA CHARSCREEN+610      ;
                LDA #$5D                ;
                STA CHARSCREEN+628      ;
                JMP PALNTSC_DONE        ;

PAL_VALUES      LDA #$00                ;1 second requires 985248 cycles of the clock (on a PAL system)
                STA TMR_A_HI            ;985248 =   256 *  3848
                LDA #$FF                ;         $00FF * $0F08
                STA TMR_A_LO            ;       timer-A * timer-B
                LDA #$0F                ;
                STA TMR_B_HI            ;
                LDA #$08                ;
                STA TMR_B_LO            ;
                LDA #$5C                ;
                STA CHARSCREEN+570      ;
                LDA #$5D                ;
                STA CHARSCREEN+588      ;
PALNTSC_DONE                            ;

                ;-----------------------

                LDA #%11110110          ;bitmap is not relevant
                                        ;$D018 = %xxxx010x -> charsetmem is at bank+$1800 ($4000 + $1800 = $5800)
                STA $D018               ;$D018 = %1111xxxx -> screenmem is at bank+$3c00  ($4000 + $3C00 = $7C00);
                                        ;screen mem is where the text on the screen is defined

                LDA #$01                ;
                STA $D019               ;the only VIC interrupt we allow to occur is the raster interrupt
                STA $D01A               ;interrupt enabled

                LDA #$35                ;disable kernal (allowing us to run interrupts in the shortest possible way)
                STA $1                  ;

              ;;;  CLI                     ;enable interrupts

                LDA $DD02               ;set video bank selection for the VIC
                ORA #3                  ;
                STA $DD02               ;

                LDA $DD00               ;
                AND #%11111100          ;
                ORA #2                  ;#2 selects VIC to use bank-1 ($4000-$7FFF)
                STA $DD00               ;


                LDA #0                  ;set background and border colour
                STA BORDER              ;poke x in 53280,x (bordercolour)
                STA BACKGROUND          ;poke x in 53281,x (background-0)
                LDA #1
                STA BACKGROUND1
                LDA #2
                STA BACKGROUND2
                LDA #3
                STA BACKGROUND3

                LDA DATA_BIT_6510       ;
                AND #%11011111          ;motor control line is set to 0 (MOTOR is now ON (6 volt))
                STA DATA_BIT_6510       ;

                ;All information is encoded in form of pulses width a defined width which are sampled by the CIA /FLAG and measured by the timer
                ;There are three types of pulses: short (352 us), medium (512 us) and long (672 us).
                ;To measure the lengths of the pulses, we must use a CIA timer (because the screen causes bad lines, screwing ony kind of loopcounter, so hardware timer is required).
                ;These timer(s) count cycles of the C64 base clock (roughly microseconds), counting down from a a 16 bit start value. Since the pulse lengths are above
                ;255, this would require us to compare 16 bit measurement results, but since microseconds are too high a resolution for tapes anyway, we can just as well only count
                ;units of 8 clock cycles this matches the resolution of the .TAP� archival format.
                ;This can be done by having timer A continuously count down from 7 to 0 (at the speed of the system clock) and having timer B count underruns of timer A:

                ;Configure CIA timer for measurement of the duration of the tape pulses (this is just a free running countdown timer)
                ;this time is also used for the 2400bits/sec baudrate generator of the bit banged RS232 signal for SSSSAM (speech synth on user port)
                LDA #7          ;divide timer B by 8
                LDX #0          ;
                STA $DD04       ;timer-A low byte
                STX $DD05       ;timer-A high byte
                LDA #%00010001  ;
                STA $DD0e       ;start timer A
                LDX #$FF        ;always start timer B from $FFFF
                STX $DD06       ;timer-B low byte
                STX $DD07       ;timer-B high byte              

SETCHARCOLORS   LDY #$0                         ;load the colors as used by the characterset 
SETCHARCOL_LP   LDA COL_MAIN,Y                  ;
                STA COLORSCREEN+0,Y             ;
                LDA COL_MAIN+256,Y              ;
                STA COLORSCREEN+256,Y           ;
                LDA COL_MAIN+512,Y              ;
                STA COLORSCREEN+512,Y           ;
                LDA COL_MAIN+768,Y              ;
                STA COLORSCREEN+768,Y           ;
                INY                             ;
                BNE SETCHARCOL_LP               ;

                LDA #0                          ;set background and border colour
                STA BACKGROUND                  ;poke x in 53281,x (background-0)

                ;-----------------------------

PREPARE_SIDDAC  LDA #$FF                ;we must boost the digi...
                STA $D406               ;this way is sounds just as "good"" on
                LDA #$49                ;the new 8580 as on the old 6581
                STA $D404               ;
                LDA #$FF                ;Setting more voices gives the digi a substantial extra boost
                STA $D406               ;
                STA $D40D               ;
                STA $D414               ;
                LDA #$49                ;
                STA $D404               ;
                STA $D40B               ;
                STA $D412               ;

                LDA #$0F    ;volume max (we will change it, but this is the starting value)
                STA $D418


;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                                              MAIN
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

    

MAIN
                LDA #$00                ;clear previous measurement value
                STA FREQCNT              ;
                STA FREQCNT+1            ;

                ;----------------------
                ;wait for trigger
                LDA #$10                ;bit-4 needs to be tested
TRIGGER_START   BIT $DC0D               ;wait for pulse (bit-4 of $DC0D)
                BEQ TRIGGER_START       ;


                ;----------------------
                ;reset timer
                ;Configure CIA timer for measurement of the duration of the tape pulses (this is just a free running countdown timer)

                LDA #0          ;stop timer-A and B
                STA $DD0E       ;before we start messing with the settings
                STA $DD0F       ;

                LDA #$0         ;
                STA $DD0D       ;clear all timer related flags


                LDA TMR_B_LO    ;timer B start value
                STA $DD06       ;timer-B low byte
                LDA TMR_B_HI    ;timer B start value
                STA $DD07       ;timer-B high byte              
                LDA #%01011001  ;
                STA $DD0F       ;start timer-B

                LDA TMR_A_LO    ;effectively multiply timer B by 15 (or make it run 15 times slower depending how you look at it)
                STA $DD04       ;timer-A low byte
                LDA TMR_A_HI    ;
                STA $DD05       ;timer-A high byte
                LDA #%00010001  ;
                STA $DD0E       ;start timer A

 

                ;----------------------
                ;count the pulses for exactly ... sec.
                ;
FREQCOUNTER     
FREQ_CNT_00     LDA #$10                ;bit-4 needs to be tested
                BIT $DC0D               ;wait for pulse (bit-4 of $DC0D)
                BEQ FREQ_CNT_01         ;
                INC BORDER              ;
                DEC BORDER              ;

                ;LDA $D418               ;if we toggle volume between max and min
                ;EOR #$0F                ;we get the max. volume with the least distortion
                ;STA $D418               ;BUT at the cost of reducing the audio frequency by 50%!!

                LDA #$0F
                STA $D418               ;BUT at the cost of reducing the audio frequency by 50%!!

                CLC                     ;
                LDA #1                  ;
                ADC FREQCNT             ;count pulse (16-bit counter)
                STA FREQCNT             ;
                BCC FREQ_CNT_01         ;
                INC FREQCNT+1           ;
           LDA FREQCNT+1
           CMP #$14
           BCS ERROR               ;When the 16-bit counter passes the $1400 value, we have encountered an error!!!
                                   ;this happens because the timer wasn't detected, ned to figure this one out!!!

FREQ_CNT_01     LDA #$00                ;
                STA $D418               ;

                LDA #02                 ;when bit 1 is set, the timer-B has experienced an underflow (a.k.a. has reached value 0)
                BIT $DD0D               ;
                BEQ FREQ_CNT_00         ;


                ;----------------------
                ;update display

                LDY #0                  ;clear previous measurement by creating an empty line
                LDA #$18                ;
CLEAR_LP        STA $5C00,Y             ;
                INY                     ;
                CPY #200                ;
                BNE CLEAR_LP            ;

                LDY FREQCNT             ;copy low-byte of value to be converted
                STY HTD_HEX             ;into a decimal printable value
                LDY FREQCNT+1           ;
                STY HTD_HEX+1           ;
                JSR HTD                 ;convert hex. to dec.

                LDX #15                 ;
                LDY #8                  ;
                JSR SET_CURSOR          ;

                LDA HTD_DEC+2           ;A-register must contain value to be printed
                JSR PRINT_HEX           ;the print routine is called
                LDA HTD_DEC+1           ;
                JSR PRINT_HEX           ;
                LDA HTD_DEC+0           ;
                JSR PRINT_HEX           ;


;                CPY ... ; check if value is within allowable range
 ;               B.. ERROR               ;
        ;        LDA #$FF                ;draw the indicator (a horizontal 1x8 pixel line)
         ;       STA $5C00,Y             ;onto the line

                JMP MAIN                ;

                ;------------------

ERROR           INC CHARSCREEN+200      ;draw a character to indicate that we've reached this point
                JMP MAIN                ;                

                ;------------------

FREQCNT         BYTE $00, $00   ;here is the 16-bit frequency counter value stored

;-------------------------------------------------------------------------------
;source: https://codebase64.org/doku.php?id=base:more_hexadecimal_to_decimal_conversion
;
; Convert an 16 bit binary value to BCD
;
; This function converts a 16 bit binary value into a 24 bit BCD. It
; works by transferring one bit a time from the source and adding it
; into a BCD value that is being doubled on each iteration. As all the
; arithmetic is being done in BCD the result is a binary to decimal
; conversion. All conversions take 915 clock cycles.
;
; See BINBCD8 for more details of its operation.
;
; Andrew Jacobs, 28-Feb-2004

HTD

BINBCD16        SED             ; Switch to decimal mode
                LDA #0          ; Ensure the result is clear
                STA HTD_DEC+0
                STA HTD_DEC+1
                STA HTD_DEC+2
                LDX #16         ; The number of source bits
                
CNVBIT          ASL HTD_HEX+0       ; Shift out one bit
                ROL HTD_HEX+1
                LDA HTD_DEC+0       ; And add into result
                ADC HTD_DEC+0
                STA HTD_DEC+0
                LDA HTD_DEC+1       ; propagating any carry
                ADC HTD_DEC+1
                STA HTD_DEC+1
                LDA HTD_DEC+2       ; ... thru whole result
                ADC HTD_DEC+2
                STA HTD_DEC+2
                DEX             ; And repeat for next bit
                BNE CNVBIT
                CLD             ; Back to binary
                
                RTS
                
                
HTD_HEX         BYTE 0, 0 
HTD_DEC         BYTE 0, 0, 0


;-------------------------------------------------------------------------------
; this routine will print the value in A as a 2 digit hexadecimal value
;        LDA #value                      ;A-register must contain value to be printed
;        JSR PRINT_HEX     ;the print routine is called
;...............................................................................
PRINT_HEX       PHA                     ;save A to stack
                AND #$F0                ;mask out low nibble
                LSR A                   ;shift to the right
                LSR A                   ;
                LSR A                   ;
                LSR A                   ;
                TAX                     ;
                LDA HEXTABLE,X          ;convert using table                                 
                JSR PRINT_CHAR          ;print character to screen

                PLA                     ;retrieve A from stack
                AND #$0F                ;mask out high nibble
                TAX                     ;
                LDA HEXTABLE,X          ;convert using table                                 
                JSR PRINT_CHAR          ;print character to screen
 
                RTS                     ;

HEXTABLE        TEXT '0123456789abcdef'                 

;-------------------------------------------------------------------------------
;call this routine as described below:
;
;               LDA #character          ;character is stored in Accumulator
;               JSR PRINT_CHAR          ;character is printed to screen, cursor is incremented by one
; also affects Y
; note: when the character value is 0 there is nothing printed but we do increment the cursor by one
;...............................................................................
PRINT_CHAR      BEQ PRINT_NOTHING       ;when the value = 0, we print nothing but we do increment the cursor by one
                LDY #00                 ;
                STA (CHAR_ADDR),Y       ;character read from string (stored in A) is now written to screen memory (see C64 manual appendix E for screen display codes)

                ;increment character pointer
PRINT_NOTHING   CLC                     ;
                LDA #$01                ;add 1
                ADC CHAR_ADDR           ;                        
                STA CHAR_ADDR           ;
                LDA #$00                ;
                ADC CHAR_ADDR+1         ;add carry... and viola, we have a new cursor position (memory location where next character will be printed)
                STA CHAR_ADDR+1         ;

                RTS                     ;

;-------------------------------------------------------------------------------
; The first location of the charsecreen (topleft) is defined as coordinate 0,0
; Use this routine before calling a PRINT related routine
;               LDX CURSOR_Y;.. chars from the top of the defined screen area
;               LDY CURSOR_X;.. chars from the left of the defined screen area
;               JSR SET_CURSOR
;...............................................................................

SET_CURSOR      LDA #<CHARSCREEN        ;
                STA CHAR_ADDR           ;store base address (low byte)
                LDA #>CHARSCREEN        ;
                STA CHAR_ADDR+1         ;store base address (high byte)

                ;calculate exact value based on the requested X and Y coordinate
                CLC                     ;
                TXA                     ;add  value in X register (to calculate the new X position of cursor)
                ADC CHAR_ADDR           ;                        
                STA CHAR_ADDR           ;
                LDA #$00                ;
                ADC CHAR_ADDR+1         ;add carry
                STA CHAR_ADDR+1         ;

SET_CURS_CHR_LP CPY #00                 ;
                BEQ SET_CURS_END        ;when Y is zero, calculation is done
                CLC                     ;clear carry for the upcoming "ADC CHAR_ADDR"

                LDA #40                 ;add  40 (which is the number of characters per line for most commodore computers) to calculate the new Y position of cursor
                ADC CHAR_ADDR           ;                        
                STA CHAR_ADDR           ;
                LDA #$00                ;
                ADC CHAR_ADDR+1         ;add carry... and viola, we have a new cursor position (memory location where next character will be printed)
                STA CHAR_ADDR+1         ;
                DEY                     ;
                JMP SET_CURS_CHR_LP     ;

SET_CURS_END    RTS                     ;





